@@ -58,6 +58,8 @@ module Distribution.Client.ProjectOrchestration (
     reportTargetSelectorProblems,
     resolveTargets,
     TargetsMap,
+    allTargetSelectors,
+    uniqueTargetSelectors,
     TargetSelector(..),
     TargetImplicitCwd(..),
     PackageId,
@@ -111,7 +113,6 @@ import qualified Distribution.Client.ProjectPlanning as ProjectPlanning
 import           Distribution.Client.ProjectPlanning.Types
 import           Distribution.Client.ProjectBuilding
 import           Distribution.Client.ProjectPlanOutput
-import           Distribution.Client.RebuildMonad ( runRebuild )
 
 import           Distribution.Client.TargetProblem
                    ( TargetProblem (..) )
@@ -119,7 +120,10 @@ import           Distribution.Client.Types
                    ( GenericReadyPackage(..), UnresolvedSourcePackage
                    , PackageSpecifier(..)
                    , SourcePackageDb(..)
-                   , WriteGhcEnvironmentFilesPolicy(..) )
+                   , WriteGhcEnvironmentFilesPolicy(..)
+                   , PackageLocation(..)
+                   , DocsResult(..)
+                   , TestsResult(..) )
 import           Distribution.Solver.Types.PackageIndex
                    ( lookupPackageName )
 import qualified Distribution.Client.InstallPlan as InstallPlan
@@ -128,12 +132,20 @@ import           Distribution.Client.TargetSelector
                    , ComponentKind(..), componentKind
                    , readTargetSelectors, reportTargetSelectorProblems )
 import           Distribution.Client.DistDirLayout
-import           Distribution.Client.Config (getCabalDir)
+
+import           Distribution.Client.BuildReports.Anonymous (cabalInstallID)
+import qualified Distribution.Client.BuildReports.Anonymous as BuildReports
+import qualified Distribution.Client.BuildReports.Storage as BuildReports
+         ( storeLocal )
+
+import           Distribution.Client.HttpUtils
 import           Distribution.Client.Setup hiding (packageName)
 import           Distribution.Compiler
                    ( CompilerFlavor(GHC) )
 import           Distribution.Types.ComponentName
                    ( componentNameString )
+import           Distribution.Types.InstalledPackageInfo
+                   ( InstalledPackageInfo )
 import           Distribution.Types.UnqualComponentName
                    ( UnqualComponentName, packageNameToUnqualComponentName )
 
@@ -149,20 +161,24 @@ import           Distribution.Simple.Flag
 import qualified Distribution.Simple.Setup as Setup
 import           Distribution.Simple.Command (commandShowOptions)
 import           Distribution.Simple.Configure (computeEffectiveProfiling)
-
+import           Distribution.Simple.PackageIndex (InstalledPackageIndex)
 import           Distribution.Simple.Utils
-                   ( die', warn, notice, noticeNoWrap, debugNoWrap, createDirectoryIfMissingVerbose )
+                   ( die', warn, notice, noticeNoWrap, debugNoWrap, createDirectoryIfMissingVerbose, ordNub )
 import           Distribution.Verbosity
 import           Distribution.Version
                    ( mkVersion )
 import           Distribution.Simple.Compiler
-                   ( compilerCompatVersion, showCompilerId
+                   ( compilerCompatVersion, showCompilerId, compilerId, compilerInfo
                    , OptimisationLevel(..))
+import           Distribution.Utils.NubList
+                   ( fromNubList )
+import           Distribution.System
+                   ( Platform(Platform) )
 
 import qualified Data.List.NonEmpty as NE
 import qualified Data.Set as Set
 import qualified Data.Map as Map
-import           Control.Exception (assert)
+import           Control.Exception ( assert )
 #ifdef MIN_VERSION_unix
 import           System.Posix.Signals (sigKILL, sigSEGV)
 #endif
@@ -170,7 +186,7 @@ import           System.Posix.Signals (sigKILL, sigSEGV)
 
 -- | Tracks what command is being executed, because we need to hide this somewhere
 -- for cases that need special handling (usually for error reporting).
-data CurrentCommand = InstallCommand | HaddockCommand | OtherCommand
+data CurrentCommand = InstallCommand | HaddockCommand | BuildCommand | ReplCommand | OtherCommand
                     deriving (Show, Eq)
 
 -- | This holds the context of a project prior to solving: the content of the
@@ -182,7 +198,8 @@ data ProjectBaseContext = ProjectBaseContext {
        projectConfig  :: ProjectConfig,
        localPackages  :: [PackageSpecifier UnresolvedSourcePackage],
        buildSettings  :: BuildTimeSettings,
-       currentCommand :: CurrentCommand
+       currentCommand :: CurrentCommand,
+       installedPackages :: Maybe InstalledPackageIndex
      }
 
 establishProjectBaseContext
@@ -195,7 +212,7 @@ establishProjectBaseContext verbosity cliConfig currentCommand = do
     establishProjectBaseContextWithRoot verbosity cliConfig projectRoot currentCommand
   where
     mprojectFile   = Setup.flagToMaybe projectConfigProjectFile
-    ProjectConfigShared { projectConfigProjectFile } = projectConfigShared cliConfig
+    ProjectConfigShared { projectConfigProjectFile} = projectConfigShared cliConfig
 
 -- | Like 'establishProjectBaseContext' but doesn't search for project root.
 establishProjectBaseContextWithRoot
@@ -205,12 +222,15 @@ establishProjectBaseContextWithRoot
     -> CurrentCommand
     -> IO ProjectBaseContext
 establishProjectBaseContextWithRoot verbosity cliConfig projectRoot currentCommand = do
-    cabalDir <- getCabalDir
-
     let distDirLayout  = defaultDistDirLayout projectRoot mdistDirectory
 
+    httpTransport <- configureTransport verbosity
+                     (fromNubList . projectConfigProgPathExtra $ projectConfigShared cliConfig)
+                     (flagToMaybe . projectConfigHttpTransport $ projectConfigBuildOnly cliConfig)
+
     (projectConfig, localPackages) <-
       rebuildProjectConfig verbosity
+                           httpTransport
                            distDirLayout
                            cliConfig
 
@@ -225,9 +245,9 @@ establishProjectBaseContextWithRoot verbosity cliConfig projectRoot currentComma
         mlogsDir = Setup.flagToMaybe projectConfigLogsDir
     mstoreDir <- sequenceA $ makeAbsolute
                  <$> Setup.flagToMaybe projectConfigStoreDir
-    let cabalDirLayout = mkCabalDirLayout cabalDir mstoreDir mlogsDir
+    cabalDirLayout <- mkCabalDirLayout mstoreDir mlogsDir
 
-        buildSettings = resolveBuildTimeSettings
+    let  buildSettings = resolveBuildTimeSettings
                           verbosity cabalDirLayout
                           projectConfig
 
@@ -241,11 +261,13 @@ establishProjectBaseContextWithRoot verbosity cliConfig projectRoot currentComma
       projectConfig,
       localPackages,
       buildSettings,
-      currentCommand
+      currentCommand,
+      installedPackages
     }
   where
     mdistDirectory = Setup.flagToMaybe projectConfigDistDir
     ProjectConfigShared { projectConfigDistDir } = projectConfigShared cliConfig
+    installedPackages = Nothing
 
 
 -- | This holds the context between the pre-build, build and post-build phases.
@@ -290,7 +312,8 @@ withInstallPlan
       distDirLayout,
       cabalDirLayout,
       projectConfig,
-      localPackages
+      localPackages,
+      installedPackages
     }
     action = do
     -- Take the project configuration and make a plan for how to build
@@ -302,6 +325,7 @@ withInstallPlan
                          distDirLayout cabalDirLayout
                          projectConfig
                          localPackages
+                         installedPackages
     action elaboratedPlan elaboratedShared
 
 runProjectPreBuildPhase
@@ -315,7 +339,8 @@ runProjectPreBuildPhase
       distDirLayout,
       cabalDirLayout,
       projectConfig,
-      localPackages
+      localPackages,
+      installedPackages
     }
     selectPlanSubset = do
     -- Take the project configuration and make a plan for how to build
@@ -327,6 +352,7 @@ runProjectPreBuildPhase
                          distDirLayout cabalDirLayout
                          projectConfig
                          localPackages
+                         installedPackages
 
     -- The plan for what to do is represented by an 'ElaboratedInstallPlan'
 
@@ -401,7 +427,7 @@ runProjectPostBuildPhase _ ProjectBaseContext{buildSettings} _ _
   = return ()
 
 runProjectPostBuildPhase verbosity
-                         ProjectBaseContext {..} ProjectBuildContext {..}
+                         ProjectBaseContext {..} bc@ProjectBuildContext {..}
                          buildOutcomes = do
     -- Update other build artefacts
     -- TODO: currently none, but could include:
@@ -422,6 +448,7 @@ runProjectPostBuildPhase verbosity
           projectConfigWriteGhcEnvironmentFilesPolicy . projectConfigShared
           $ projectConfig
 
+        shouldWriteGhcEnvironment :: Bool
         shouldWriteGhcEnvironment =
           case fromFlagOrDefault NeverWriteGhcEnvironmentFiles
                writeGhcEnvFilesPolicy
@@ -439,6 +466,9 @@ runProjectPostBuildPhase verbosity
                                      elaboratedShared
                                      postBuildStatus
 
+    -- Write the build reports
+    writeBuildReports buildSettings bc elaboratedPlanToExecute buildOutcomes
+
     -- Finally if there were any build failures then report them and throw
     -- an exception to terminate the program
     dieOnBuildFailures verbosity currentCommand elaboratedPlanToExecute buildOutcomes
@@ -473,7 +503,15 @@ runProjectPostBuildPhase verbosity
 -- possible to for different selectors to match the same target. This extra
 -- information is primarily to help make helpful error messages.
 --
-type TargetsMap = Map UnitId [(ComponentTarget, [TargetSelector])]
+type TargetsMap = Map UnitId [(ComponentTarget, NonEmpty TargetSelector)]
+
+-- | Get all target selectors.
+allTargetSelectors :: TargetsMap -> [TargetSelector]
+allTargetSelectors = concatMap (NE.toList . snd) . concat . Map.elems
+
+-- | Get all unique target selectors.
+uniqueTargetSelectors :: TargetsMap -> [TargetSelector]
+uniqueTargetSelectors = ordNub . allTargetSelectors
 
 -- | Given a set of 'TargetSelector's, resolve which 'UnitId's and
 -- 'ComponentTarget's they ought to refer to.
@@ -529,7 +567,7 @@ resolveTargets selectPackageTargets selectComponentTarget
                  -> TargetsMap
     mkTargetsMap targets =
         Map.map nubComponentTargets
-      $ Map.fromListWith (++)
+      $ Map.fromListWith (<>)
           [ (uid, [(ct, ts)])
           | (ts, cts) <- targets
           , (uid, ct) <- cts ]
@@ -659,37 +697,50 @@ type AvailableTargetsMap k = Map k [AvailableTarget (UnitId, ComponentName)]
 availableTargetIndexes :: ElaboratedInstallPlan -> AvailableTargetIndexes
 availableTargetIndexes installPlan = AvailableTargetIndexes{..}
   where
+    availableTargetsByPackageIdAndComponentName ::
+      Map (PackageId, ComponentName)
+          [AvailableTarget (UnitId, ComponentName)]
     availableTargetsByPackageIdAndComponentName =
       availableTargets installPlan
 
+    availableTargetsByPackageId ::
+      Map PackageId [AvailableTarget (UnitId, ComponentName)]
     availableTargetsByPackageId =
                   Map.mapKeysWith
                     (++) (\(pkgid, _cname) -> pkgid)
                     availableTargetsByPackageIdAndComponentName
       `Map.union` availableTargetsEmptyPackages
 
+    availableTargetsByPackageName ::
+      Map PackageName [AvailableTarget (UnitId, ComponentName)]
     availableTargetsByPackageName =
       Map.mapKeysWith
         (++) packageName
         availableTargetsByPackageId
 
+    availableTargetsByPackageNameAndComponentName ::
+      Map (PackageName, ComponentName)
+          [AvailableTarget (UnitId, ComponentName)]
     availableTargetsByPackageNameAndComponentName =
       Map.mapKeysWith
         (++) (\(pkgid, cname) -> (packageName pkgid, cname))
         availableTargetsByPackageIdAndComponentName
 
+    availableTargetsByPackageNameAndUnqualComponentName ::
+      Map (PackageName, UnqualComponentName)
+          [AvailableTarget (UnitId, ComponentName)]
     availableTargetsByPackageNameAndUnqualComponentName =
       Map.mapKeysWith
         (++) (\(pkgid, cname) -> let pname  = packageName pkgid
                                      cname' = unqualComponentName pname cname
                                   in (pname, cname'))
         availableTargetsByPackageIdAndComponentName
-     where
-       unqualComponentName ::
-         PackageName -> ComponentName -> UnqualComponentName
-       unqualComponentName pkgname =
-           fromMaybe (packageNameToUnqualComponentName pkgname)
-         . componentNameString
+      where
+        unqualComponentName ::
+          PackageName -> ComponentName -> UnqualComponentName
+        unqualComponentName pkgname =
+            fromMaybe (packageNameToUnqualComponentName pkgname)
+          . componentNameString
 
     -- Add in all the empty packages. These do not appear in the
     -- availableTargetsByComponent map, since that only contains
@@ -722,26 +773,24 @@ filterTargetsKindWith p ts =
         , p (componentKind cname) ]
 
 selectBuildableTargets :: [AvailableTarget k] -> [k]
-selectBuildableTargets ts =
-    [ k | AvailableTarget _ _ (TargetBuildable k _) _ <- ts ]
+selectBuildableTargets = selectBuildableTargetsWith (const True)
+
+zipBuildableTargetsWith :: (TargetRequested -> Bool)
+                        -> [AvailableTarget k] -> [(k, AvailableTarget k)]
+zipBuildableTargetsWith p ts =
+    [ (k, t) | t@(AvailableTarget _ _ (TargetBuildable k req) _) <- ts, p req ]
 
 selectBuildableTargetsWith :: (TargetRequested -> Bool)
                           -> [AvailableTarget k] -> [k]
-selectBuildableTargetsWith p ts =
-    [ k | AvailableTarget _ _ (TargetBuildable k req) _ <- ts, p req ]
+selectBuildableTargetsWith p = map fst . zipBuildableTargetsWith p
 
 selectBuildableTargets' :: [AvailableTarget k] -> ([k], [AvailableTarget ()])
-selectBuildableTargets' ts =
-    (,) [ k | AvailableTarget _ _ (TargetBuildable k _) _ <- ts ]
-        [ forgetTargetDetail t
-        | t@(AvailableTarget _ _ (TargetBuildable _ _) _) <- ts ]
+selectBuildableTargets' = selectBuildableTargetsWith' (const True)
 
 selectBuildableTargetsWith' :: (TargetRequested -> Bool)
                            -> [AvailableTarget k] -> ([k], [AvailableTarget ()])
-selectBuildableTargetsWith' p ts =
-    (,) [ k | AvailableTarget _ _ (TargetBuildable k req) _ <- ts, p req ]
-        [ forgetTargetDetail t
-        | t@(AvailableTarget _ _ (TargetBuildable _ req) _) <- ts, p req ]
+selectBuildableTargetsWith' p =
+  (fmap . map) forgetTargetDetail . unzip . zipBuildableTargetsWith p
 
 
 forgetTargetDetail :: AvailableTarget k -> AvailableTarget ()
@@ -817,26 +866,26 @@ printPlan verbosity
           ProjectBaseContext {
             buildSettings = BuildTimeSettings{buildSettingDryRun},
             projectConfig = ProjectConfig {
+              projectConfigAllPackages =
+                  PackageConfig {packageConfigOptimization = globalOptimization},
               projectConfigLocalPackages =
-                  PackageConfig {packageConfigOptimization}
-            }
+                  PackageConfig {packageConfigOptimization = localOptimization}
+            },
+            currentCommand
           }
           ProjectBuildContext {
             elaboratedPlanToExecute = elaboratedPlan,
             elaboratedShared,
             pkgsBuildStatus
           }
-
-  | null pkgs
-  = notice verbosity "Up to date"
-
-  | otherwise
-  = noticeNoWrap verbosity $ unlines $
+  | null pkgs && currentCommand == BuildCommand
+    = notice verbosity "Up to date"
+  | not (null pkgs) = noticeNoWrap verbosity $ unlines $
       (showBuildProfile ++ "In order, the following "
        ++ wouldWill ++ " be built"
        ++ ifNormal " (use -v for more details)" ++ ":")
     : map showPkgAndReason pkgs
-
+  | otherwise = return ()
   where
     pkgs = InstallPlan.executionOrder elaboratedPlan
 
@@ -865,6 +914,7 @@ printPlan verbosity
         in "(" ++ showBuildStatus buildStatus ++ ")"
       ]
 
+    showComp :: ElaboratedConfiguredPackage -> ElaboratedComponent -> String
     showComp elab comp =
         maybe "custom" prettyShow (compComponentName comp) ++
         if Map.null (elabInstantiatedWith elab)
@@ -879,6 +929,7 @@ printPlan verbosity
     nonDefaultFlags elab =
       elabFlagAssignment elab `diffFlagAssignment` elabFlagDefaults elab
 
+    showTargets :: ElaboratedConfiguredPackage -> String
     showTargets elab
       | null (elabBuildTargets elab) = ""
       | otherwise
@@ -887,6 +938,7 @@ printPlan verbosity
                             | t <- elabBuildTargets elab ]
         ++ ")"
 
+    showConfigureFlags :: ElaboratedConfiguredPackage -> String
     showConfigureFlags elab =
         let fullConfigureFlags
               = setupHsConfigureFlags
@@ -920,6 +972,7 @@ printPlan verbosity
             (Setup.configureCommand (pkgConfigCompilerProgs elaboratedShared))
             partialConfigureFlags
 
+    showBuildStatus :: BuildStatus -> String
     showBuildStatus status = case status of
       BuildStatusPreExisting -> "existing package"
       BuildStatusInstalled   -> "already installed"
@@ -937,6 +990,7 @@ printPlan verbosity
           BuildReasonEphemeralTargets -> "ephemeral targets"
       BuildStatusUpToDate {} -> "up to date" -- doesn't happen
 
+    showMonitorChangedReason :: MonitorChangedReason a -> String
     showMonitorChangedReason (MonitoredFileChanged file) =
       "file " ++ file ++ " changed"
     showMonitorChangedReason (MonitoredValueChanged _)   = "value changed"
@@ -944,15 +998,69 @@ printPlan verbosity
     showMonitorChangedReason  MonitorCorruptCache        =
       "cannot read state cache"
 
+    showBuildProfile :: String
     showBuildProfile = "Build profile: " ++ unwords [
       "-w " ++ (showCompilerId . pkgConfigCompiler) elaboratedShared,
-      "-O" ++  (case packageConfigOptimization of
+      "-O" ++  (case globalOptimization <> localOptimization of -- if local is not set, read global
                 Setup.Flag NoOptimisation      -> "0"
                 Setup.Flag NormalOptimisation  -> "1"
                 Setup.Flag MaximumOptimisation -> "2"
                 Setup.NoFlag                   -> "1")]
       ++ "\n"
 
+
+writeBuildReports :: BuildTimeSettings -> ProjectBuildContext -> ElaboratedInstallPlan -> BuildOutcomes -> IO ()
+writeBuildReports settings buildContext plan buildOutcomes = do
+  let plat@(Platform arch os) = pkgConfigPlatform . elaboratedShared $ buildContext
+      comp = pkgConfigCompiler . elaboratedShared $ buildContext
+      getRepo (RepoTarballPackage r _ _) = Just r
+      getRepo _ = Nothing
+      fromPlanPackage (InstallPlan.Configured pkg) (Just result) =
+            let installOutcome = case result of
+                   Left bf -> case buildFailureReason bf of
+                      DependentFailed p -> BuildReports.DependencyFailed p
+                      DownloadFailed _  -> BuildReports.DownloadFailed
+                      UnpackFailed _ -> BuildReports.UnpackFailed
+                      ConfigureFailed _ -> BuildReports.ConfigureFailed
+                      BuildFailed _ -> BuildReports.BuildFailed
+                      TestsFailed _ -> BuildReports.TestsFailed
+                      InstallFailed _ -> BuildReports.InstallFailed
+
+                      ReplFailed _ -> BuildReports.InstallOk
+                      HaddocksFailed _ -> BuildReports.InstallOk
+                      BenchFailed _ -> BuildReports.InstallOk
+
+                   Right _br -> BuildReports.InstallOk
+
+                docsOutcome = case result of
+                   Left bf -> case buildFailureReason bf of
+                      HaddocksFailed _ -> BuildReports.Failed
+                      _ -> BuildReports.NotTried
+                   Right br -> case buildResultDocs br of
+                      DocsNotTried -> BuildReports.NotTried
+                      DocsFailed -> BuildReports.Failed
+                      DocsOk -> BuildReports.Ok
+
+                testsOutcome = case result of
+                   Left bf -> case buildFailureReason bf of
+                      TestsFailed _ -> BuildReports.Failed
+                      _ -> BuildReports.NotTried
+                   Right br -> case buildResultTests br of
+                      TestsNotTried -> BuildReports.NotTried
+                      TestsOk -> BuildReports.Ok
+
+            in Just $ (BuildReports.BuildReport (packageId pkg) os arch (compilerId comp) cabalInstallID (elabFlagAssignment pkg) (map packageId $ elabLibDependencies pkg) installOutcome docsOutcome testsOutcome, getRepo . elabPkgSourceLocation $ pkg) -- TODO handle failure log files?
+      fromPlanPackage _ _ = Nothing
+      buildReports = mapMaybe (\x -> fromPlanPackage x (InstallPlan.lookupBuildOutcome x buildOutcomes)) $ InstallPlan.toList plan
+
+
+  BuildReports.storeLocal (compilerInfo comp)
+                          (buildSettingSummaryFile settings)
+                          buildReports
+                          plat
+  -- Note this doesn't handle the anonymous build reports set by buildSettingBuildReports but those appear to not be used or missed from v1
+  -- The usage pattern appears to be that rather than rely on flags to cabal to send build logs to the right place and package them with reports, etc, it is easier to simply capture its output to an appropriate handle.
+
 -- | If there are build failures then report them and throw an exception.
 --
 dieOnBuildFailures :: Verbosity -> CurrentCommand
@@ -991,9 +1099,11 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
          | let mentionDepOf = verbosity <= normal
          , (pkg, failureClassification) <- failuresClassification ]
   where
+    failures :: [(UnitId, BuildFailure)]
     failures =  [ (pkgid, failure)
                 | (pkgid, Left failure) <- Map.toList buildOutcomes ]
 
+    failuresClassification :: [(ElaboratedConfiguredPackage, BuildFailurePresentation)]
     failuresClassification =
       [ (pkg, classifyBuildFailure failure)
       | (pkgid, failure) <- failures
@@ -1004,6 +1114,7 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
            maybeToList (InstallPlan.lookup plan pkgid)
       ]
 
+    dieIfNotHaddockFailure :: Verbosity -> String -> IO ()
     dieIfNotHaddockFailure
       | currentCommand == HaddockCommand            = die'
       | all isHaddockFailure failuresClassification = warn
@@ -1040,12 +1151,13 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
     --    detail itself (e.g. ghc reporting errors on stdout)
     --  - then we do not report additional error detail or context.
     --
+    isSimpleCase :: Bool
     isSimpleCase
       | [(pkgid, failure)] <- failures
       , [pkg]              <- rootpkgs
       , installedUnitId pkg == pkgid
       , isFailureSelfExplanatory (buildFailureReason failure)
-      , currentCommand /= InstallCommand
+      , currentCommand `notElem` [InstallCommand, BuildCommand, ReplCommand]
       = True
       | otherwise
       = False
@@ -1053,6 +1165,7 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
     -- NB: if the Setup script segfaulted or was interrupted,
     -- we should give more detailed information.  So only
     -- assume that exit code 1 is "pedestrian failure."
+    isFailureSelfExplanatory :: BuildFailureReason -> Bool
     isFailureSelfExplanatory (BuildFailed e)
       | Just (ExitFailure 1) <- fromException e = True
 
@@ -1061,11 +1174,15 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
 
     isFailureSelfExplanatory _                  = False
 
+    rootpkgs :: [ElaboratedConfiguredPackage]
     rootpkgs =
       [ pkg
       | InstallPlan.Configured pkg <- InstallPlan.toList plan
       , hasNoDependents pkg ]
 
+    ultimateDeps
+      :: UnitId
+      -> [InstallPlan.GenericPlanPackage InstalledPackageInfo ElaboratedConfiguredPackage]
     ultimateDeps pkgid =
         filter (\pkg -> hasNoDependents pkg && installedUnitId pkg /= pkgid)
                (InstallPlan.reverseDependencyClosure plan [pkgid])
@@ -1073,11 +1190,13 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
     hasNoDependents :: HasUnitId pkg => pkg -> Bool
     hasNoDependents = null . InstallPlan.revDirectDeps plan . installedUnitId
 
+    renderFailureDetail :: Bool -> ElaboratedConfiguredPackage -> BuildFailureReason -> String
     renderFailureDetail mentionDepOf pkg reason =
         renderFailureSummary mentionDepOf pkg reason ++ "."
      ++ renderFailureExtraDetail reason
      ++ maybe "" showException (buildFailureException reason)
 
+    renderFailureSummary :: Bool -> ElaboratedConfiguredPackage -> BuildFailureReason -> String
     renderFailureSummary mentionDepOf pkg reason =
         case reason of
           DownloadFailed  _ -> "Failed to download " ++ pkgstr
@@ -1099,6 +1218,7 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
                    then renderDependencyOf (installedUnitId pkg)
                    else ""
 
+    renderFailureExtraDetail :: BuildFailureReason -> String
     renderFailureExtraDetail (ConfigureFailed _) =
       " The failure occurred during the configure step."
     renderFailureExtraDetail (InstallFailed   _) =
@@ -1106,6 +1226,7 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
     renderFailureExtraDetail _                   =
       ""
 
+    renderDependencyOf :: UnitId -> String
     renderDependencyOf pkgid =
       case ultimateDeps pkgid of
         []         -> ""
@@ -1167,6 +1288,7 @@ dieOnBuildFailures verbosity currentCommand plan buildOutcomes
              ++ show e
 #endif
 
+    buildFailureException :: BuildFailureReason -> Maybe SomeException
     buildFailureException reason =
       case reason of
         DownloadFailed  e -> Just e
@@ -1193,21 +1315,14 @@ data BuildFailurePresentation =
 establishDummyProjectBaseContext
   :: Verbosity
   -> ProjectConfig
+     -- ^ Project configuration including the global config if needed
   -> DistDirLayout
      -- ^ Where to put the dist directory
   -> [PackageSpecifier UnresolvedSourcePackage]
      -- ^ The packages to be included in the project
   -> CurrentCommand
   -> IO ProjectBaseContext
-establishDummyProjectBaseContext verbosity cliConfig distDirLayout localPackages currentCommand = do
-    cabalDir <- getCabalDir
-
-    globalConfig <- runRebuild ""
-                  $ readGlobalConfig verbosity
-                  $ projectConfigConfigFile
-                  $ projectConfigShared cliConfig
-    let projectConfig = globalConfig <> cliConfig
-
+establishDummyProjectBaseContext verbosity projectConfig distDirLayout localPackages currentCommand = do
     let ProjectConfigBuildOnly {
           projectConfigLogsDir
         } = projectConfigBuildOnly projectConfig
@@ -1218,11 +1333,14 @@ establishDummyProjectBaseContext verbosity cliConfig distDirLayout localPackages
 
         mlogsDir = flagToMaybe projectConfigLogsDir
         mstoreDir = flagToMaybe projectConfigStoreDir
-        cabalDirLayout = mkCabalDirLayout cabalDir mstoreDir mlogsDir
 
+    cabalDirLayout <- mkCabalDirLayout mstoreDir mlogsDir
+
+    let buildSettings :: BuildTimeSettings
         buildSettings = resolveBuildTimeSettings
                           verbosity cabalDirLayout
                           projectConfig
+        installedPackages = Nothing
 
     return ProjectBaseContext {
       distDirLayout,
@@ -1230,7 +1348,8 @@ establishDummyProjectBaseContext verbosity cliConfig distDirLayout localPackages
       projectConfig,
       localPackages,
       buildSettings,
-      currentCommand
+      currentCommand,
+      installedPackages
     }
 
 establishDummyDistDirLayout :: Verbosity -> ProjectConfig -> FilePath -> IO DistDirLayout
